#include <EEPROM.h>
#include <Wire.h>
#include <Time.h>
#include <LiquidCrystal_I2C.h>
#include <eRCaGuy_analogReadXXbit.h>


// create custom characters for LCD-bargraph
byte level1[8] = { 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b11111};
byte level2[8] = { 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b11111, 0b11111};
byte level3[8] = { 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111};
byte level4[8] = { 0b00000, 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b11111};
byte level5[8] = { 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111};
byte level6[8] = { 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111};
byte level7[8] = { 0b00000, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111};
byte level8[8] = { 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111};

//                     addr, en,rw,rs,d4,d5,d6,d7,bl,blpol
LiquidCrystal_I2C lcd1(0x26, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);  // top - set the LCD1 I2C address
LiquidCrystal_I2C lcd2(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);  // bottom - set the LCD2 I2C address (pin 1 of PCF8574 connected to ground)


//instantiate an object of this library class; call it "adc"
eRCaGuy_analogReadXXbit adc;

//Global constants
//constants required to determine the voltage at the pin
const float MAX_READING_10_bit = 1023.0;
//const float MAX_READING_11_bit = 2046.0;
const float MAX_READING_12_bit = 4092.0;
//const float MAX_READING_13_bit = 8184.0;
const float MAX_READING_14_bit = 16368.0;
//const float MAX_READING_15_bit = 32736.0;
//const float MAX_READING_16_bit = 65472.0;
//const float MAX_READING_17_bit = 130944.0;
//const float MAX_READING_18_bit = 261888.0;
//const float MAX_READING_19_bit = 523776.0;
//const float MAX_READING_20_bit = 1047552.0;
//const float MAX_READING_21_bit = 2095104.0;

float voltage0 = 0;      //A0
float current0 = 0;      //A1
float power0 = 0;
float resistance0 = 0;
float temperature0 = 0;  //A2
uint8_t fanpower0 = 0;   //pin 11
float voltage1 = 0;      //A3
float current1 = 0;      //A4
float power1 = 0;
float resistance1 = 0;
float temperature1 = 0;  //A5
uint8_t fanpower1 = 0;   //pin 12
float voltage2 = 0;      //A6
float current2 = 0;      //A7
float power2 = 0;
//float resistance2 = 0;   //I want to show mAh instead of resistance on the bottom left side because this will be used for USB charging.
float mAh2 = 0;
int mAh2mem = 0;
float mAh2sum = 0;
float temperature2 = 0;  //A8
uint8_t fanpower2 = 0;   //pin 9
float voltage3 = 0;      //A9
float current3 = 0;      //A10
float power3 = 0;
float resistance3 = 0;
float temperature3 = 0;  //A11
uint8_t fanpower3 = 0;   //pin 10

uint8_t hours = 0;
uint8_t mins = 0;
uint8_t lastSecond;
time_t startTime = 0;

int resetmAh = 2;        //pin 2 is the reset switch for mAh stats

void setup()   /*----( SETUP: RUNS ONCE )----*/
{
  lcd1.begin(20, 4);        // initialize the top LCD (1) for 20 chars 4 lines, turn on backlight and create custom chars for vertical bargraph.
  lcd1.createChar(1, level1);
  lcd1.createChar(2, level2);
  lcd1.createChar(3, level3);
  lcd1.createChar(4, level4);
  lcd1.createChar(5, level5);
  lcd1.createChar(6, level6);
  lcd1.createChar(7, level7);
  lcd1.createChar(8, level8);

  lcd2.begin(20, 4);        // initialize the bottom LCD (2) for 20 chars 4 lines, turn on backlight and create custom chars for vertical bargraph.
  lcd2.createChar(1, level1);
  lcd2.createChar(2, level2);
  lcd2.createChar(3, level3);
  lcd2.createChar(4, level4);
  lcd2.createChar(5, level5);
  lcd2.createChar(6, level6);
  lcd2.createChar(7, level7);
  lcd2.createChar(8, level8);

  analogReference(DEFAULT);
  //analogReference(EXTERNAL);  //use  this only if you know what you are doing!
  //analogReference(INTERNAL2V56);
  //analogReference(INTERNAL1V1);

  // do not use timer0, it will change delay time!!
  TCCR1B = TCCR1B & 0b11111000 | 0x01; // set timer1 (Pin 11 und 12) to 31300Hz
  TCCR2B = TCCR2B & 0b11111000 | 0x01; // set timer2 (Pin 9 und 10) to 31300Hz

mAh2mem=eepromReadInt(0); //get saved data from adress 0; max. 65280


pinMode(resetmAh, INPUT);
digitalWrite(2, HIGH);   // internal Pull up to 5V


    // print persistant charactars to LCD #################################################################################################################  print persistant charactors to LCD
    //LCD 1 top left side
    lcd1.setCursor(7, 0);
    lcd1.print("V");
    lcd1.setCursor(7, 1);
    lcd1.print("A");
    lcd1.setCursor(7, 2);
    lcd1.print("W");
    lcd1.setCursor(7, 3);
    lcd1.write(0b11110100); //OMEGA
    
    //LCD 1 top right side
    lcd1.setCursor(17, 0);
    lcd1.print("V");
    lcd1.setCursor(17, 1);
    lcd1.print("A");
    lcd1.setCursor(17, 2);
    lcd1.print("W");
    lcd1.setCursor(17, 3);
    lcd1.write(0b11110100); //OMEGA
    
    //LCD 2 bottom left side
    lcd2.setCursor(7, 0);
    lcd2.print("V");
    lcd2.setCursor(7, 1);
    lcd2.print("A");
    lcd2.setCursor(7, 2);
    lcd2.print("W");
    lcd2.setCursor(6, 3);
    lcd2.print("mAh");
    
    //LCD 2 bottom right side
    lcd2.setCursor(17, 0);
    lcd2.print("V");
    lcd2.setCursor(17, 1);
    lcd2.print("A");
    lcd2.setCursor(17, 2);
    lcd2.print("W");
    lcd2.setCursor(17, 3);
    lcd2.write(0b11110100); //OMEGA
    
}/*--(end setup )---*/


void loop()   /*----( LOOP: RUNS CONSTANTLY )----*/
{
  {
    // if button is pressed the mAh stats will be set to zero
    if (digitalRead(resetmAh) == LOW) {
    mAh2 = 0;
    mAh2mem = 0;
    mAh2sum = 0;
    }
    
    // get all the analog reads an calculate  #################################################################################################################  get all the analog reads an calculate
    //LCD 1 top left side
    voltage0 = ((adc.analogReadXXbit(A0, 12, 20) / 4095)*5);
    current0 = adc.analogReadXXbit(A1, 12, 20) / 433;
    power0 = voltage0 * current0;
    resistance0 = voltage0 / (current0 + 0.0001);
    temperature0 = (analogRead(A2) / 7) + 6;
    //fancontrol 0
    fanpower0 = analogRead(A2) / 4;
    analogWrite(11, fanpower0);
    
    //LCD 1 top right side
    voltage1 = ((adc.analogReadXXbit(A3, 12, 20) / 4095)*5);
    current1 = adc.analogReadXXbit(A4, 12, 20) / 988;
    power1 = voltage1 * current1;
    resistance1 = voltage1 / (current1 + 0.0001);
    temperature1 = (analogRead(A5) / 7) + 6;
        //fancontrol 1
    fanpower1 = analogRead(A5) / 4;
    analogWrite(12, fanpower1);
    
    //LCD 2 bottom left side
    voltage2 = adc.analogReadXXbit(A6, 10, 20) / 101;
    current2 = (adc.analogReadXXbit(A7, 12, 20) / 100) - 20.45;
    if (current2 < 0.001)
      current2=0;
    power2 = voltage2 * current2;
    //mAh2
    time_t t = now() - startTime;
    uint8_t sec = second(t);
    if (sec != lastSecond) {
      lastSecond = sec;
      hours = hour(t);
      mins = minute(t);
      mAh2 += (current2 / 3.6);  //current is in A an we increase mA/h every second => 1000ma / (60*60) = 3,6
      if (sec = lastSecond)
      {
      }
    }
    mAh2sum = mAh2 + mAh2mem;
    eepromWriteInt(0, mAh2sum);
    
    //LCD 2 bottom right side
    voltage3 = ((adc.analogReadXXbit(A9, 12, 20) / 4095)*5);
    current3 = adc.analogReadXXbit(A10, 12, 20) / 410;
    power3 = voltage3 * current3;
    resistance3 = voltage3 / (current3 + 0.0001);
    temperature3 = (analogRead(A11) / 7) + 6;
    //fancontrol 3
    fanpower3 = analogRead(A11) / 4;
    analogWrite(10, fanpower3);
    
    
    //print the results on the LCD    
    //LCD 1 top ****************************************************************************************************************************
    //start left side
    //voltage 0
    lcd1.setCursor(1, 0);
    if (voltage0 < 10)
      lcd1.print(" ");
    lcd1.print(voltage0);

    //current 0
    lcd1.setCursor(1, 1);
    if (current0 < 10)
      lcd1.print(" ");
    lcd1.print(current0, 2);

    //power 0
    lcd1.setCursor(0, 2);
    if (power0 < 10)
      lcd1.print(" ");
    if (power0 < 100)
      lcd1.print(" ");
    lcd1.print(power0, 2);

    //resistance 0
    lcd1.setCursor(0, 3);
    if (resistance0 < 10)
      lcd1.print(" ");
    if (resistance0 < 100)
      lcd1.print(" ");
    if (resistance0 < 9999)
      lcd1.print(resistance0, 2);
    else
      lcd1.print("  -.-- ");
    if (resistance0 < 1000)
      lcd1.print(" ");

    //temp bar 0
    lcd1.setCursor(9, 0);
    if (temperature0 >= 124) {
      lcd1.write(8);
    } else if (temperature0 >= 121) {
      lcd1.write(7);
    } else if (temperature0 >= 118) {
      lcd1.write(6);
    } else if (temperature0 >= 115) {
      lcd1.write(5);
    } else if (temperature0 >= 112) {
      lcd1.write(4);
    } else if (temperature0 >= 109) {
      lcd1.write(3);
    } else if (temperature0 >= 106) {
      lcd1.write(2);
    } else if (temperature0 >= 103) {
      lcd1.write(1);
    } else if (temperature0 <= 100) {
      lcd1.print(" ");
    }
    lcd1.setCursor(9, 1);
    if (temperature0 >= 100) {
      lcd1.write(8);
    } else if (temperature0 >= 97) {
      lcd1.write(7);
    } else if (temperature0 >= 94) {
      lcd1.write(6);
    } else if (temperature0 >= 91) {
      lcd1.write(5);
    } else if (temperature0 >= 88) {
      lcd1.write(4);
    } else if (temperature0 >= 85) {
      lcd1.write(3);
    } else if (temperature0 >= 82) {
      lcd1.write(2);
    } else if (temperature0 >= 79) {
      lcd1.write(1);
    } else if (temperature0 <= 76) {
      lcd1.print(" ");
    }
    lcd1.setCursor(9, 2);
    if (temperature0 >= 76) {
      lcd1.write(8);
    } else if (temperature0 >= 73) {
      lcd1.write(7);
    } else if (temperature0 >= 70) {
      lcd1.write(6);
    } else if (temperature0 >= 67) {
      lcd1.write(5);
    } else if (temperature0 >= 64) {
      lcd1.write(4);
    } else if (temperature0 >= 61) {
      lcd1.write(3);
    } else if (temperature0 >= 58) {
      lcd1.write(2);
    } else if (temperature0 >= 55) {
      lcd1.write(1);
    } else if (temperature0 <= 52) {
      lcd1.print(" ");
    }
    lcd1.setCursor(9, 3);
    if (temperature0 >= 52) {
      lcd1.write(8);
    } else if (temperature0 >= 49) {
      lcd1.write(7);
    } else if (temperature0 >= 46) {
      lcd1.write(6);
    } else if (temperature0 >= 43) {
      lcd1.write(5);
    } else if (temperature0 >= 40) {
      lcd1.write(4);
    } else if (temperature0 >= 37) {
      lcd1.write(3);
    } else if (temperature0 >= 34) {
      lcd1.write(2);
    } else if (temperature0 <= 33.9) {
      lcd1.write(1);
    }

    //start right side
    //voltage 1
    lcd1.setCursor(11, 0);
    if (voltage1 < 10)
      lcd1.print(" ");
    lcd1.print(voltage1);

    //current 1
    lcd1.setCursor(11, 1);
    if (current1 < 10)
      lcd1.print(" ");
    lcd1.print(current1, 3);

    //power 1
    lcd1.setCursor(10, 2);
    if (power1 < 10)
      lcd1.print(" ");
    if (power1 < 100)
      lcd1.print(" ");
    lcd1.print(power1, 2);

    //resistance 1
    lcd1.setCursor(10, 3);
    if (resistance1 < 10)
      lcd1.print(" ");
    if (resistance1 < 100)
      lcd1.print(" ");
    if (resistance1 < 9999)
      lcd1.print(resistance1, 2);
    else
      lcd1.print("  -.-- ");
    if (resistance1 < 1000)
      lcd1.print(" ");

    //temp bar1
    lcd1.setCursor(19, 0);
    if (temperature1 >= 124) {
      lcd1.write(8);
    } else if (temperature1 >= 121) {
      lcd1.write(7);
    } else if (temperature1 >= 118) {
      lcd1.write(6);
    } else if (temperature1 >= 115) {
      lcd1.write(5);
    } else if (temperature1 >= 112) {
      lcd1.write(4);
    } else if (temperature1 >= 109) {
      lcd1.write(3);
    } else if (temperature1 >= 106) {
      lcd1.write(2);
    } else if (temperature1 >= 103) {
      lcd1.write(1);
    } else if (temperature1 <= 100) {
      lcd1.print(" ");
    }
    lcd1.setCursor(19, 1);
    if (temperature1 >= 100) {
      lcd1.write(8);
    } else if (temperature1 >= 97) {
      lcd1.write(7);
    } else if (temperature1 >= 94) {
      lcd1.write(6);
    } else if (temperature1 >= 91) {
      lcd1.write(5);
    } else if (temperature1 >= 88) {
      lcd1.write(4);
    } else if (temperature1 >= 85) {
      lcd1.write(3);
    } else if (temperature1 >= 82) {
      lcd1.write(2);
    } else if (temperature1 >= 79) {
      lcd1.write(1);
    } else if (temperature1 <= 76) {
      lcd1.print(" ");
    }
    lcd1.setCursor(19, 2);
    if (temperature1 >= 76) {
      lcd1.write(8);
    } else if (temperature1 >= 73) {
      lcd1.write(7);
    } else if (temperature1 >= 70) {
      lcd1.write(6);
    } else if (temperature1 >= 67) {
      lcd1.write(5);
    } else if (temperature1 >= 64) {
      lcd1.write(4);
    } else if (temperature1 >= 61) {
      lcd1.write(3);
    } else if (temperature1 >= 58) {
      lcd1.write(2);
    } else if (temperature1 >= 55) {
      lcd1.write(1);
    } else if (temperature1 <= 52) {
      lcd1.print(" ");
    }
    lcd1.setCursor(19, 3);
    if (temperature1 >= 52) {
      lcd1.write(8);
    } else if (temperature1 >= 49) {
      lcd1.write(7);
    } else if (temperature1 >= 46) {
      lcd1.write(6);
    } else if (temperature1 >= 43) {
      lcd1.write(5);
    } else if (temperature1 >= 40) {
      lcd1.write(4);
    } else if (temperature1 >= 37) {
      lcd1.write(3);
    } else if (temperature1 >= 34) {
      lcd1.write(2);
    } else if (temperature1 <= 33.9) {
      lcd1.write(1);
    }
    
    
    //LCD 2 bottom ****************************************************************************************************************************
    //start left side
    //voltage 2
    lcd2.setCursor(2, 0);
    lcd2.print(voltage2);

    //current 2
    lcd2.setCursor(2, 1);
    lcd2.print(current2, 2);

    //power 2
    lcd2.setCursor(0, 2);
    if (power2 < 10)
      lcd2.print(" ");
    if (power2 < 100)
      lcd2.print(" ");
    lcd2.print(power2, 2);

    //mAh2
    lcd2.setCursor(0, 3);
    if (mAh2sum <= 9.445)
      lcd2.print(" ");
    if (mAh2sum < 99.445)
      lcd2.print(" ");
    if (mAh2sum < 999.445)
      lcd2.print(" ");
    if (mAh2sum < 9999.445)
      lcd2.print(" ");
    lcd2.print(mAh2sum, 0);
    if (mAh2sum > 99999)
      lcd2.print("  -.--");

/*
    //temp bar 2
    lcd2.setCursor(9, 0);
    temperature2 = (analogRead(A8) / 7) + 6;
    if (temperature2 >= 124) {
      lcd2.write(8);
    } else if (temperature2 >= 121) {
      lcd2.write(7);
    } else if (temperature2 >= 118) {
      lcd2.write(6);
    } else if (temperature2 >= 115) {
      lcd2.write(5);
    } else if (temperature2 >= 112) {
      lcd2.write(4);
    } else if (temperature2 >= 109) {
      lcd2.write(3);
    } else if (temperature2 >= 106) {
      lcd2.write(2);
    } else if (temperature2 >= 103) {
      lcd2.write(1);
    } else if (temperature2 <= 100) {
      lcd2.print(" ");
    }
    lcd2.setCursor(9, 1);
    if (temperature2 >= 100) {
      lcd2.write(8);
    } else if (temperature2 >= 97) {
      lcd2.write(7);
    } else if (temperature2 >= 94) {
      lcd2.write(6);
    } else if (temperature2 >= 91) {
      lcd2.write(5);
    } else if (temperature2 >= 88) {
      lcd2.write(4);
    } else if (temperature2 >= 85) {
      lcd2.write(3);
    } else if (temperature2 >= 82) {
      lcd2.write(2);
    } else if (temperature2 >= 79) {
      lcd2.write(1);
    } else if (temperature2 <= 76) {
      lcd2.print(" ");
    }
    lcd2.setCursor(9, 2);
    if (temperature2 >= 76) {
      lcd2.write(8);
    } else if (temperature2 >= 73) {
      lcd2.write(7);
    } else if (temperature2 >= 70) {
      lcd2.write(6);
    } else if (temperature2 >= 67) {
      lcd2.write(5);
    } else if (temperature2 >= 64) {
      lcd2.write(4);
    } else if (temperature2 >= 61) {
      lcd2.write(3);
    } else if (temperature2 >= 58) {
      lcd2.write(2);
    } else if (temperature2 >= 55) {
      lcd2.write(1);
    } else if (temperature2 <= 52) {
      lcd2.print(" ");
    }
    lcd2.setCursor(9, 3);
    if (temperature2 >= 52) {
      lcd2.write(8);
    } else if (temperature2 >= 49) {
      lcd2.write(7);
    } else if (temperature2 >= 46) {
      lcd2.write(6);
    } else if (temperature2 >= 43) {
      lcd2.write(5);
    } else if (temperature2 >= 40) {
      lcd2.write(4);
    } else if (temperature2 >= 37) {
      lcd2.write(3);
    } else if (temperature2 >= 34) {
      lcd2.write(2);
    } else if (temperature2 <= 33.9) {
      lcd2.write(1);
    }

    //fancontrol 2 PWM pin9
    fanpower2 = analogRead(A8) / 4;
    analogWrite(9, fanpower0);
*/


    //start right side
    //voltage 3
    lcd2.setCursor(11, 0);
    if (voltage3 < 10)
      lcd2.print(" ");
    lcd2.print(voltage3);

    //current 3
    lcd2.setCursor(11, 1);
    if (current3 < 10)
      lcd2.print(" ");
    lcd2.print(current3, 2);

    //power 3
    lcd2.setCursor(10, 2);
    if (power3 < 10)
      lcd2.print(" ");
    if (power3 < 100)
      lcd2.print(" ");
    lcd2.print(power3, 2);

    //resistance 3
    lcd2.setCursor(10, 3);
    if (resistance3 < 10)
      lcd2.print(" ");
    if (resistance3 < 100)
      lcd2.print(" ");
    if (resistance3 < 9999)
      lcd2.print(resistance3, 2);
    else
      lcd2.print("  -.-- ");
    if (resistance3 < 1000)
      lcd2.print(" ");

    //temp bar3
    lcd2.setCursor(19, 0);
    if (temperature3 >= 124) {
      lcd2.write(8);
    } else if (temperature3 >= 121) {
      lcd2.write(7);
    } else if (temperature3 >= 118) {
      lcd2.write(6);
    } else if (temperature3 >= 115) {
      lcd2.write(5);
    } else if (temperature3 >= 112) {
      lcd2.write(4);
    } else if (temperature3 >= 109) {
      lcd2.write(3);
    } else if (temperature3 >= 106) {
      lcd2.write(2);
    } else if (temperature3 >= 103) {
      lcd2.write(1);
    } else if (temperature3 <= 100) {
      lcd2.print(" ");
    }
    lcd2.setCursor(19, 1);
    if (temperature3 >= 100) {
      lcd2.write(8);
    } else if (temperature3 >= 97) {
      lcd2.write(7);
    } else if (temperature3 >= 94) {
      lcd2.write(6);
    } else if (temperature3 >= 91) {
      lcd2.write(5);
    } else if (temperature3 >= 88) {
      lcd2.write(4);
    } else if (temperature3 >= 85) {
      lcd2.write(3);
    } else if (temperature3 >= 82) {
      lcd2.write(2);
    } else if (temperature3 >= 79) {
      lcd2.write(1);
    } else if (temperature3 <= 76) {
      lcd2.print(" ");
    }
    lcd2.setCursor(19, 2);
    if (temperature3 >= 76) {
      lcd2.write(8);
    } else if (temperature3 >= 73) {
      lcd2.write(7);
    } else if (temperature3 >= 70) {
      lcd2.write(6);
    } else if (temperature3 >= 67) {
      lcd2.write(5);
    } else if (temperature3 >= 64) {
      lcd2.write(4);
    } else if (temperature3 >= 61) {
      lcd2.write(3);
    } else if (temperature3 >= 58) {
      lcd2.write(2);
    } else if (temperature3 >= 55) {
      lcd2.write(1);
    } else if (temperature3 <= 52) {
      lcd2.print(" ");
    }
    lcd2.setCursor(19, 3);
    if (temperature3 >= 52) {
      lcd2.write(8);
    } else if (temperature3 >= 49) {
      lcd2.write(7);
    } else if (temperature3 >= 46) {
      lcd2.write(6);
    } else if (temperature3 >= 43) {
      lcd2.write(5);
    } else if (temperature3 >= 40) {
      lcd2.write(4);
    } else if (temperature3 >= 37) {
      lcd2.write(3);
    } else if (temperature3 >= 34) {
      lcd2.write(2);
    } else if (temperature3 <= 33.9) {
      lcd2.write(1);
    }

    //delay (250);

  }

}/* --(end main loop )-- */

int eepromReadInt(int adr) {
byte low, high;
  low=EEPROM.read(adr);
  high=EEPROM.read(adr+1);
  return low + ((high << 8)&0xFF00);
}
 
void eepromWriteInt(int adr, int wert) {
byte low, high;
  low=wert&0xFF;
  high=(wert >> 8)&0xFF;
  EEPROM.write(adr, low);
  EEPROM.write(adr+1, high);
  return;
}

/* ( THE END ) */
